package dk.apaq.framework.repository;

import java.util.*;
import dk.apaq.framework.criteria.Criteria;
import dk.apaq.framework.criteria.CriteriaUtil;
import dk.apaq.framework.criteria.Limit;
import dk.apaq.framework.criteria.Rule;
import dk.apaq.framework.criteria.Rule.FiltrationItem;
import dk.apaq.framework.criteria.Sorter;
import dk.apaq.framework.criteria.rules.BeanFiltrationItem;

/**
 *
 */
public class CollectionRepository<BEANTYPE> extends BaseRepository<BEANTYPE, String> {

    private final List<BEANTYPE> beans;
    private final IdResolver<BEANTYPE> idResolver;

    private static class DefaultIdResolver<BEANTYPE> implements IdResolver<BEANTYPE> {

        private final WeakHashMap<BEANTYPE, String> idmap = new WeakHashMap<BEANTYPE, String>();

        public String getIdForBean(BEANTYPE bean) {
            String id = idmap.get(bean);
            if (id == null) {
                id = UUID.randomUUID().toString();
                idmap.put(bean, id);
            }
            return id;
        }
    }

    public interface IdResolver<BEANTYPE> {

        String getIdForBean(BEANTYPE bean);
    }

    public CollectionRepository() {
        this(null, new DefaultIdResolver<BEANTYPE>());
    }

    public CollectionRepository(IdResolver<BEANTYPE> idResolver) {
        this(null, idResolver);
    }

    public CollectionRepository(List<BEANTYPE> beans) {
        this(beans, new DefaultIdResolver<BEANTYPE>());
    }

    public CollectionRepository(List<BEANTYPE> beans, IdResolver<BEANTYPE> idResolver) {
        this.beans = beans == null ? new ArrayList<BEANTYPE>() : beans;
        this.idResolver = idResolver;
    }

    public List<BEANTYPE> getBeans() {
        return beans;
    }

    public String getIdFromEntity(BEANTYPE entity) {
        return idResolver.getIdForBean(entity);
    }

    public BEANTYPE save(BEANTYPE entity) {
        if (!beans.contains(entity)) {
            fireOnBeforeCreate(entity);
            beans.add(entity);
            String id = idResolver.getIdForBean(entity);
            fireOnCreate(id, entity);
        } else {
            String id = idResolver.getIdForBean(entity);
            fireOnBeforeUpdate(null, entity);
            fireOnUpdate(id, entity);
        }
        return entity;
    }

    public void delete(BEANTYPE entity) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void deleteById(String id) {
        fireOnBeforeDelete(id);

        Iterator<BEANTYPE> it = beans.iterator();

        while (it.hasNext()) {
            BEANTYPE b = it.next();
            if (id.equals(idResolver.getIdForBean(b))) {
                it.remove();
                break;
            }
        }

        fireOnDelete(id);
    }

    public BEANTYPE findOne(String id) {
        fireOnBeforeRead(id);
        for (BEANTYPE current : beans) {
            String currentId = idResolver.getIdForBean(current);
            if (id.equals(currentId)) {
                fireOnRead(id, current);
                return current;
            }
        }
        fireOnRead(id, null);
        return null;
    }

    public List<BEANTYPE> findAll(Criteria criteria) {
        List<String> idlist = findAllIds(criteria);
        List<BEANTYPE> beanlist = new ArrayList<BEANTYPE>();

        for (String id : idlist) {
            beanlist.add(findOne(id));
        }
        return beanlist;

    }

    public List<String> findAllIds(Criteria criteria) {
        Rule rule = criteria == null ? null : criteria.getRule();
        Limit limit = criteria == null ? null : criteria.getLimit();
        Sorter sorter = criteria == null ? null : criteria.getSorter();
        fireOnBeforeList(criteria);

        List<BEANTYPE> list = CriteriaUtil.filterCollection(beans, criteria);
        List<String> idlist = new ArrayList<String>();
        for (BEANTYPE e : list) {
            String id = idResolver.getIdForBean(e);
            idlist.add(id);
        }

        return idlist;
    }
}
